/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.client.servlet;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import de.hendriklipka.ldse.common.*;
import de.hendriklipka.ldse.client.*;

public class LDSESearch extends HttpServlet
{
    protected ClientConfig _config=null;
    protected Vector _quotes=new Vector(5,5);
    protected Random _random=new Random();
    
    public void LDSESearch()
    {
    }
    public void init(ServletConfig config)
         throws ServletException
    {
//        super.init(config);
        // Konfiguration einlesen und so zusammenbauen, dass das Config-Objekt
        // was damit anfangen kann...
        String port=config.getInitParameter("de.hendriklipka.ldse.server.port");
        String address=config.getInitParameter("de.hendriklipka.ldse.server.address");
        String[] args={"server="+address,"port="+port};
        _config =new ClientConfig(args);
        String file=config.getInitParameter("de.hendriklipka.ldse.quotes");
        try
        {
            BufferedReader br=new BufferedReader(new FileReader(file),1);
            _quotes=new Vector(10000,1000);
            while (br.ready())
            {
                String line=br.readLine();
                _quotes.addElement(line);
            }
        }
        catch (Exception ex) {}
    }

    public void doRequest(HttpServletRequest sreq, HttpServletResponse sres, Hashtable args) throws ServletException, IOException
    {
        Searcher s=new Searcher(sreq,sres,args);
        s.doRequest();
    }

    public void doPost(HttpServletRequest sreq, HttpServletResponse sres) 
        throws ServletException, IOException
    {
        Hashtable args=new Hashtable();
        Enumeration params = sreq.getParameterNames();
        while (params.hasMoreElements())
        {
            String name=(String)params.nextElement();
            String value=sreq.getParameter(name);
            if (name.equals("WORDS"))
            {
                args.put("words",value);
            }
            else if (name.equals("MAXDOC"))
            {
                args.put("maxdoc",value);
            }
            else if (name.equals("MINLEVEL"))
            {
                args.put("minlevel",value);
            }
            else if (name.equals("EXTENDED"))
            {
                args.put("doextended",value);
            }
        }
        doRequest(sreq,sres,args);
    }
    
    public void doGet(HttpServletRequest sreq, HttpServletResponse sres) 
        throws ServletException, IOException
    {   
        Hashtable args=new Hashtable();
        // Parsing des QueryString
        String qs=sreq.getQueryString();

        // String:
        // Parameter werden durch & getrennt
        // einzelne Worte sind mit + verbunden
        // Sonderzeichen werden als %xx (Hex) dargestellt
        // MultiSelect wird abgebildet, indem der Name 2-mal auftaucht
        StringTokenizer st=new StringTokenizer(qs,"&"); 
        while (st.hasMoreElements())
        {
            try
            {
                String element=(String)st.nextElement();
                StringTokenizer st2=new StringTokenizer(element,"=");
                String name=(String)st2.nextElement();
                String value=(String)st2.nextElement();
                //ToDo
                // Value-String parsen und %xx ersetzen
                value=value.replace('+',' ');
                String v="";
                int pos=value.indexOf('%');
                while (-1!=pos)
                {
                    v+=value.substring(0,pos);
                    String ch=value.substring(pos+1,pos+3);
                    value=value.substring(pos+3);
                    pos=value.indexOf('%');
                    int i=Integer.parseInt(ch,16);
                    v+=(char)i;
                }
                v+=value;
                if (name.equals("WORDS"))
                {
                    args.put("words",v);
                }
                else if (name.equals("MAXDOC"))
                {
                    args.put("maxdoc",v);
                }
                else if (name.equals("MINLEVEL"))
                {
                    args.put("minlevel",v);
                }
                else if (name.equals("EXTENDED"))
                {
                    args.put("doextended",v);
                }
            }
            catch (NoSuchElementException ex) {}
        }
        doRequest(sreq,sres,args);
    }

    /* ------------------------------------------------------------ */
    public String getServletInfo()
    {
        return "Search Servlet";
    }

    /* ------------------------------------------------------------ */
    public synchronized void destroy()
    {
//        Code.debug("Destroyed");
    }
    
    private class Searcher implements ResultListener
    {
        protected PrintWriter _pout=null;
        protected int _num=0;
        protected int _lNum=0;
        protected Hashtable _args;
        String _words=null;
        String _maxDoc=null;
        String _minLevel=null;
        String _doExtended=null;
        
        public Searcher(HttpServletRequest sreq, HttpServletResponse sres, Hashtable args) throws ServletException
        {
            sres.setContentType("text/html");
            _args=args;
            try
            {
                _pout = sres.getWriter();
            }
            catch (IOException ex)
            {
                throw new ServletException(ex);
            }
        }
        
        public void doRequest() throws ServletException, IOException
        {
            // Parameter der Suche
            _words=(String)_args.get("words");
            if (null==_words)
                _words="";
            _maxDoc=(String)_args.get("maxdoc");
            if (null==_maxDoc)
                _maxDoc="0";
            _minLevel=(String)_args.get("minlevel");
            if (null==_minLevel)
                _minLevel="0";
            _doExtended=(String)_args.get("doextended");
            if (null==_doExtended)
                _doExtended="NO";    
                
            addHeader();
            _pout.flush();
            // Suche durchfuehren
            // 1. Generator holen
            QueryGenerator qg=null;
            if (_doExtended.equals("YES"))
            {
                qg=new ComplexQueryGenerator(_words);
            }
            else
            {
                qg=new SimpleQueryGenerator(_words);
            }
            // 2. Limits auslesen
            int maxDocs=0;
            double mLevel=0;
            try
            {
                maxDocs=Integer.parseInt(_maxDoc);
            }
            catch (NumberFormatException ex) {}
                
            try
            {
                mLevel=Double.valueOf(_minLevel).doubleValue();
            }
            catch (NumberFormatException ex) {}
            // 3. Limits setzen
            qg.setLimits(maxDocs,mLevel);
            try
            {
                // 4. Verbindung herstellen
                ServerConnection sc=new ServerConnection(_config);
                sc.init();
                Writer wr=sc.getWriter();
                Reader r=sc.getReader();
                // wenn Erfolg
                if ((null!=wr)&&(null!=r))
                {
                    // 5. Anfrage schreiben
                    qg.writeQuery(wr);
                    // 6. Parser erzeugen
                    ResultParser rp=new ResultParser();
                    rp.init(r);
                    // 7. Listener hinzufuegen
                    rp.addResultListener(this);
                    // 8. Ergebnis einlesen
                    rp.readResult();
                }
            }
            catch (Throwable ex)
            {
                printError(ex);
                ex.printStackTrace();
            }
                
            addFooter();
        
            _pout.flush();
        }

        // ein einzelnes Ergebnis hinzufuegen
        protected void addNewResult(ResultDescriptor rd, int num)
        {
            _pout.println("<BR><TABLE BORDER=\"0\" COLSPAN=\"1\" VALIGN=\"TOP\">");
            // URL
            _pout.println("<TR><TD>");
            _pout.println("<A HREF=\""+rd.getURL()+"\">"+rd.getURL()+"</A>");
            _pout.println("</TD></TR>");
            // Relevanz
            _pout.println("<TR><TD>");
            _pout.println("Relevanz:"+rd.getRelevance());
            _pout.println("</TD></TR>");
            // Titel
            if (null!=rd.getTitle())
            {
                _pout.println("<TR><TD>");
                _pout.println(rd.getTitle());
                _pout.println("</TD></TR>");
            }
            // Groesse
            _pout.println("<TR><TD>");
            _pout.print("Groesse: ");
            if (0==rd.getSize())
            {
                _pout.println("0 (or unknown)");
            }
            else
            {
                _pout.println(rd.getSize()+" bytes");
            }
            _pout.println("</TD></TR>");

            _pout.println("</TABLE>");
            _pout.flush();
        }
        
        // vor dem ersten Ergebnis
        protected void preResults()
        {
            _pout.println("<BR>");
            if (0!=_quotes.size())
            {
                int i=(Math.abs(_random.nextInt()))%(_quotes.size());
                String s=(String)_quotes.elementAt(i);
                _pout.println("<TABLE BORDER=\"0\" CELLPADDING=\"2\" CELLSPACING=\"3\""+
                " COLSPAN=\"1\" VALIGN=\"BOTTOM\"><TR><TD>Fortune:</TD>"+
                "<TD>"+s+"</TD></TR></TABLE>");
            }
            _pout.println("<BR>");
            _pout.println("<H1>Search results for: "+_words+"</H1>");
            _pout.flush();
        }
        
        // nach dem letzten Ergebnis
        protected void postResults()
        {
            _pout.println("<BR>");
            _pout.println("found "+_lNum+" results");
            _pout.println("<BR>");
            _pout.println("<BR>");
            _pout.println("<FORM ACTION=\"/Search\" METHOD=\"GET\">");
            _pout.println("<TABLE BORDER=\"1\" COLSPAN=\"1\" VALIGN=\"TOP\">");
            _pout.println("<TR>");
            _pout.println("<TD>");
            _pout.println("<TABLE BORDER=\"0\" CELLPADDING=\"2\" CELLSPACING=\"3\">");
            _pout.println("<TR>");
            _pout.println("<TD>Search terms:</TD>");
            _pout.println("<TD><INPUT TYPE=\"TEXT\" NAME=\"WORDS\" SIZE=\"30\"></TD>");
            _pout.println("</TR>");
            _pout.println("<TR>");
            _pout.println("<TD>Extended search:</TD>");
            _pout.println("<TD><INPUT TYPE=\"CHECKBOX\" NAME=\"EXTENDED\" VALUE=\"YES\"></TD>");
            _pout.println("</TR>");
            _pout.println("<TR>");
            _pout.println("<TD>Maximum number of documents:</TD>");
            _pout.println("<TD><INPUT TYPE=\"TEXT\" NAME=\"MAXDOC\" SIZE=\"5\"></TD>");
            _pout.println("</TR>");
            _pout.println("<TR>");
            _pout.println("<TD>Minimal level of relevance:</TD>");
            _pout.println("<TD><INPUT TYPE=\"TEXT\" NAME=\"MINLEVEL\" SIZE=\"5\"></TD>");
            _pout.println("</TR>");
            _pout.println("<TR>");
            _pout.println("<TD>&nbsp;</TD>");
            _pout.println("<TD><INPUT TYPE=\"SUBMIT\" NAME=\"Search!\" VALUE=\"Submit\"></TD>");
            _pout.println("</TR>");
            _pout.println("</TABLE>");
            _pout.println("</TD>");
            _pout.println("</TR>");
            _pout.println("</TABLE>");
            _pout.println("</FORM>");
        }
        
        // am Anfang der Seite
        protected void addHeader()
        {
            _pout.println("<HTML>\n<HEAD><TITLE>Suchergebnisse</TITLE>\n</HEAD>\n<BODY>");
        }
        
        // am Ende der Seite
        protected void addFooter()
        {
            _pout.println("<BR>\n</BODY>\n</HTML>");
        }
        
        protected void printError(Throwable ex)
        {
            _pout.println("internal error while searching: "+ex.toString()+"<BR>");
            _pout.println("please contact system administrator");
        }
        
        public void startResult()
        {
            preResults();
        }
        
        public void addResult(ResultDescriptor rd)
        {
            _lNum++;
            addNewResult(rd,_lNum);
        }
        
        public void setResultNum(int num)
        {
            _num=num;
        }
        
        public void endResult()
        {
            postResults();
        }
        
        public int getResultNum()
        {
            return _num;
        }
    }
}
