/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.data.document;

import com.objectspace.jgl.BinaryPredicate;

public class Triplet
{
  public int num=-1;
  public int count=-1;
  public final static int _minNum=0;
  public final static int _maxNum=47+48*47+48*48*47;
  
  // Achtung: es ist SEHR unguenstig, " als erlaubtes Zeichen zu haben - das gibt Aerger mit der DB
  protected static char[] _triples={'a','b','c','d','e','f',
                                    'g','h','i','j','k','l',
                                    'm','n','o','p','q','r',
                                    's','t','u','v','w','x',
                                    'y','z','0','1','2','3',
                                    '4','5','6','7','8','9',
                                    '','','','','-','_',
                                    '&',0,'$','+','@',0}; // '.', damit Adresse als Wort erhalten bleiben
  // Lookup-tabelle, um fuer ein Char den Triplet-Wert zu erhalten
  protected static int[] _tripleVal=new int[256];
  // dazu gehoerender Initialisierer
  static
  {
    for (int i=0;i<256;i++)
    {
      _tripleVal[i]=-1;
      for (int j=0;j<48;j++)
      {
        if (i==_triples[j])
        {
          _tripleVal[i]=j;
          break;
        }
      }
    }
  }
  
  public Triplet(int n)
  {
    num=n;
    if (num>112944)
    {
//      Log.log(0,"error while creating triplet: num > 112944: "+num);
    }
    if (num<-1)
    {
//      Log.log(0,"error while creating triplet: num < -1: "+num);
    }
  }
  
  public Triplet(int n, int c)
  {
    num=n;
    count=c;
    if (num>112944)
    {
//      Log.log(0,"error while creating triplet: num > 112944: "+num);
    }
    if (num<-1)
    {
//      Log.log(0,"error while creating triplet: num < -1: "+num);
    }
  }
  
  public Triplet(char[] ca) throws Exception
  {
    if (ca.length!=3)
    {
      throw new Exception("Triplet.Triplet(char[]): length!=3");
    }
    int val=0;
    try
    {
      int i=_tripleVal[ca[0]];
      if (i==-1)
      {
        return;
      }
      val+=i;
      i=_tripleVal[ca[1]];
      if (i==-1)
      {
        return;
      }
      val+=(i*48);
      i=_tripleVal[ca[2]];
      if (i==-1)
      {
        return;
      }
      val+=(i*48*48);
    }
    catch (Exception ex)
    { // wenn Zeichen ausserhalb des Bereiches anfallen...
      return;
    }
    if (val>112944)
    {
      return;
//      Log.log(0,"error while creating triplet: num > 112944: "+num);
    }
    if (val<-1)
    {
      return;
//      Log.log(0,"error while creating triplet: num < -1: "+num);
    }
    num=val;
    count=1;
  }
  
  public int hashCode()
  {
    return num;
  }
  
  public boolean equals(Object obj)
  {
    if (!(obj instanceof Triplet))
    {
      return false;
    }
    if (num!=((Triplet)obj).num)
    {
      return false;
    }
    return true;
  }
  
  public static boolean isValidChar(char c) 
  {
    try
    {
        int i=_tripleVal[Character.toLowerCase(c)];
        return (-1!=i);
    }
    catch (Exception ex)
    {
        return false;
    }
  }
  
  public static BinaryPredicate getBinaryPredicate()
  {
    return _myBinPred;
  }
  
  protected static BinaryPredicate _myBinPred=new BinPred();
  
  static class BinPred implements BinaryPredicate
  {
    public boolean execute( Object first, Object second )
    {
      return first.equals( second );
    }
  }
}