/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.data.document;

import java.io.*;
import java.net.URL;
import java.util.zip.*;

import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.server.data.document.Document;
import de.hendriklipka.ldse.server.indexer.DocumentFactory;
import de.hendriklipka.ldse.server.data.URLInfo;
import de.hendriklipka.ldse.common.Spider;

/**
  zerlegt ein Zip-Dokument in Worte
  aus dem Namen der Datei wird der originale Namen bestimmt und versucht, dafuer
  ein Document zu erzeugen
  geht das, wird der Stream entpackt und umgeleitet
  
  das wird fuer jedes weitere enthaltene File so gemacht
*/
public class ZipDocument extends Document implements Spider
{
  protected Reader _reader=null;
  protected ZipInputStream _ziStream=null;
  protected Document _doc=null;
  protected URL _url=null;
  protected URLInfo _urlInfo;
  
  String _descWords="";
  
  public boolean init()
  {
		try
		{
		  _ziStream=new ZipInputStream(_urlInfo.getInputStreamForURL(_urlInfo._url));
		  ZipEntry ze=_ziStream.getNextEntry();

		  String n=ze.getName();
  		
      Log.log("indexer","document",2,"zip-document: new file name is "+n);
      URL u=new URL(_urlInfo._url.getProtocol(),_urlInfo._url.getHost(),_urlInfo._url.getPort(),n);
      URLInfo ui=new URLInfo();
      ui._url=u;
      ui._contextURL=_urlInfo._contextURL;
      ui._urlStr=u.toString();
      ui._spiderID=_urlInfo._spiderID;
      ui._spider=this;
		  ui._type="";
		  // damit versorgen wir das neue Dokument mit den richtigen Daten, da wir selber
		  // ein Spider sind...
		  _url=u;
		  _urlInfo=ui;
		  
		  _doc=DocumentFactory.getDocument(ui);
		  if (null!=_doc)
  		  return _doc.init();
		}
		catch (Exception ex)
		{
		  _doc=null;
		}
    return false;
  }
  
  public boolean hasMoreWords()
  {
    if (null==_doc)
    {
      return false;
    }
    boolean b=_doc.hasMoreWords();
    if (b)
      return true;
    
		try
		{
		  _descWords+=_doc.getDescriptiveWords();
		  // wenn nix mehr kommt, versuchen wir den naechsten Eintrag zu oeffnen
		  _ziStream.closeEntry();
		  ZipEntry ze=_ziStream.getNextEntry();
		  String n=ze.getName();
      Log.log("indexer","document",2,"zip-document: new file name is "+n);
      URL u=new URL(_urlInfo._url.getProtocol(),_urlInfo._url.getHost(),_urlInfo._url.getPort(),n);
		  _urlInfo._url=u;
		  _urlInfo._urlStr=u.toString();
		  _url=u;
		  
		  _doc=DocumentFactory.getDocument(_urlInfo);
		  _doc.init();
		  return hasMoreWords();
		}
		catch (Exception ex)
		{
		  _doc=null;
		  return false;
		}
  }
  
  public String nextWord()
  {
    if (null==_doc)
    {
      return null;
    }
    String s=_doc.nextWord();
    if (null!=s)
      return s;
    
		try
		{
		  _descWords+=_doc.getDescriptiveWords();
		  // wenn nix mehr kommt, versuchen wir den naechsten Eintrag zu oeffnen
		  _ziStream.closeEntry();
		  ZipEntry ze=_ziStream.getNextEntry();
		  String n=ze.getName();
      URL u=new URL(_urlInfo._url.getProtocol(),_urlInfo._url.getHost(),_urlInfo._url.getPort(),n);
		  _urlInfo._url=u;
		  _urlInfo._urlStr=u.toString();
		  _url=u;
		  
		  _doc=DocumentFactory.getDocument(_urlInfo);
		  _doc.init();
		  return nextWord();
		}
		catch (Exception ex)
		{
		  _doc=null;
		  return null;
		}
  }
  
  public String getDescriptiveWords()
  {
    return "";
  }
  
  public void finish()
  {
    try
    {
      _ziStream.close();
    }
    catch (Exception ex) {}
  }
  
  public java.io.Reader getReaderForURL(java.net.URL url) throws java.io.IOException
  {
    if (url==_url)
    {
      return new InputStreamReader(_ziStream);
    }
    return null;
  }
  
  public java.io.InputStream getInputStreamForURL(java.net.URL url) throws java.io.IOException
  {
    if (url==_url)
    {
      return _ziStream;
    }
    return null;
  }
  public void init(org.w3c.dom.Node configNode, int id){}
  public boolean hasMoreElements() { return false;}
  public URLInfo getNextElement() {return null;}
  public void start() {}
}