/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.gatherer;

import java.net.*;
import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import Acme.*;
import HTTPClient.*;

import de.hendriklipka.ldse.common.Timer;
import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.common.HTTPConnectionPool;
import de.hendriklipka.ldse.server.data.URLInfo;
import de.hendriklipka.ldse.common.Spider;

public class HTTPSpider implements Spider
{
  protected PrintStream err;
  
  protected URL url=null;
  
  // Liste aller geladenen URLs. enthaelt URLInfos
  // begrenz, weil sie eh gleich wieder rausgeworfen werden...
  protected Vector done = new Vector(500,500);
  
  // Liste alle bekannten URLs
  // alles, was hier steht, wurde entweder schon geladen oder
  // ist in einem der Threads in der ToDo-Liste
  protected Hashtable knownURLs=new Hashtable(100000,0.5f);
  
  // Liste aller laufenden Threads
  protected int _threadNum=6;
  protected SpiderRunner[] _runner=new SpiderRunner[_threadNum];
  
  protected String[] serverInclude=null;
  protected String[] serverExclude=null;
  protected String[] pathInclude=null;
  protected String[] pathExclude=null;
  protected int maxDepth=30;
  protected int _maxQueue=50;

  protected int _id=-1;
  
  // The optional authorization cookie.
  private String auth_cookie = null;
  // The Robot Exclusion checker.
  protected String _name="de.hendriklipka.ldse.Robot";
  private Acme.NoRobots noRobots = null;
  
  public void init(Node configNode, int id)
  {
    _id=id;
    NodeList nl=configNode.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node node=nl.item(i);
      if (node.getNodeType()!=node.ELEMENT_NODE)
        continue;
      String nn=node.getNodeName();
      if (nn.equals("depth"))
      {
        maxDepth=Integer.parseInt(de.hendriklipka.ldse.common.Util.getTextData(node));
        Log.log("spider","config",1,"depth="+maxDepth);
      }
      else if (nn.equals("pathinclude"))
      {
        pathInclude=de.hendriklipka.ldse.common.Util.stringToArray(de.hendriklipka.ldse.common.Util.getTextData(node));
      }
      else if (nn.equals("pathexclude"))
      {
        pathExclude=de.hendriklipka.ldse.common.Util.stringToArray(de.hendriklipka.ldse.common.Util.getTextData(node));
      }
      else if (nn.equals("serverinclude"))
      {
        serverInclude=de.hendriklipka.ldse.common.Util.stringToArray(de.hendriklipka.ldse.common.Util.getTextData(node));
      }
      else if (nn.equals("serverexclude"))
      {
        serverExclude=de.hendriklipka.ldse.common.Util.stringToArray(de.hendriklipka.ldse.common.Util.getTextData(node));
      }
      else if (nn.equals("pathinclude"))
      {
        _name=de.hendriklipka.ldse.common.Util.getTextData(node);
      }
      else if (nn.equals("maxthreads"))
      {
        // hier ist keine Synchronisation noetig, da das ohnehin nur
        // beim Hochstarten gemacht wird, und da fummelt noch keiner dran rum
        String s=de.hendriklipka.ldse.common.Util.getTextData(node);
        _threadNum=Integer.parseInt(s);
        _runner=new SpiderRunner[_threadNum];
      }
      else if (nn.equals("start"))
      {
        try
        {
          String urlStr=de.hendriklipka.ldse.common.Util.getTextData(node);
          Log.log("spider","config",1,"start="+urlStr);
          url=new URL(urlStr);
          knownURLs.put(urlStr,this);
        }
        catch (Exception ex)
        {
          Log.log("spider","error",0,ex.toString());
        }
      }
      else
      {
        Log.log("spider","error",1,"unknown node - type="+node.getNodeType()+" name="+node.getNodeName()+" value="+node.getNodeName());
      }
    }
    noRobots=new NoRobots(_name);
  }
  
  public void start()
  {
    // ersten SpiderRunner starten
    Log.log("spider","threads",2,"start: starting spider with start-url "+url);
    URLInfo ui=null;
    try
    {
      ui=new URLInfo();
      ui._url=url;
      ui._contextURL=url;
      ui._urlStr=url.toString();
      ui._spiderID=_id;
      ui._spider=this;
      boolean b=doThisUrl(ui._urlStr,url,0);
      if (!b)
      {
        Log.log("spider","threads",1,"doThisUrl says no :(");
      }
      if (!noRobots.ok(ui._url))
      {
        Log.log("spider","threads",0,"Robot Exclusion says no - finished");
        return;
      }
      SpiderRunner sr=new SpiderRunner(ui,this);
      _runner[0]=sr;
      sr.start();
    }  
    catch (Exception ex)
    {
      ex.printStackTrace(Log.getWriter("spider","error",0));
    }
  }

  // URLs, die nicht geladen werden sollen, werden auch nicht
  // gespeichert. Das spart Speicherplatz, kostet aber Rechenzeit
  protected boolean doThisUrl(String urlStr, URL _url,int depth)	
  {	
    if (null==urlStr)
    {
      return false;
    }
    // Robot Exclusion
    if (!noRobots.ok(_url))
    {
      return false;
    }
    synchronized(done)
    {
      if (done.size()>_maxQueue)
      {
        // Blockieren
        Log.log("spider","threads",1,"stopping because queue to large");
        try
        {
          done.wait();
        }
        catch (InterruptedException ex) {}
        Log.log("spider","threads",1,"restarting this thread");
      }
      // Tiefenbegrenzung
      // URL wird nicht geaddet, damit sie u.U. noch von woanders aus
      // referenziert und geladen werden kann (wenn dort die Tiefe kleiner ist)
      if (depth>maxDepth)
      {
        Log.log("spider","url",2,"doThisUrl: beyond max depth");
        return false;
      }
      // maximales Ausschliessen... 
      // Begrenzung auf Protokoll
      if (!urlStr.startsWith("http://"))
      {
        Log.log("spider","url",2,"doThisUrl: wrong protocol");
        return false;
      }
      try
      {
        // Bearbeitung serverInclude
        String host=_url.getHost();
        boolean found=false;
        if (null!=serverInclude)
        {
          for (int i=0;i<serverInclude.length;i++)
          {
            if (-1!=host.indexOf(serverInclude[i]))
            {
              found=true;
              break;
            }
          }
          if (!found&&(serverInclude.length>0))
          {
            Log.log("spider","url",2,"doThisUrl: no serverinclude");
            return false; // Server enthaelt kein Include
          }
        }
        if (null!=serverExclude)
        {
          // Bearbeitung serverExclude
          for (int i=0;i<serverExclude.length;i++)
          {
            if (-1!=host.indexOf(serverExclude[i]))
            {
              Log.log("spider","url",2,"doThisUrl: in serverexclude");
              return false;
            }
          }
        }
        
        // Bearbeitung pathInclude
        String path=_url.getFile();
        if (null!=pathInclude)
        {
          found=false; 
          for (int i=0;i<pathInclude.length;i++)
          {
            if (-1!=path.indexOf(pathInclude[i]))
            {
              found=true;
              break;
            }
          }
          if (!found&&(pathInclude.length>0))
          {
            Log.log("spider","url",2,"doThisUrl: no path include");
            return false; // Path enthaelt kein Include
          }
        }
        // Bearbeitung pathExclude
        if (null!=pathExclude)
        {
          for (int i=0;i<pathExclude.length;i++)
          {
            if (-1!=path.indexOf(pathExclude[i]))
            {
              Log.log("spider","url",2,"doThisUrl: found path exclude");
              return false;
            }
          }
        }
      }
      catch (Exception ex) 
      {
        Log.log("spider","error",1,"doThisUrl: "+ex.toString()+" -> "+urlStr);
      }
      // Abspeichern der URL nur fuer geladene URLs
      synchronized (knownURLs)
      {
        // wenn wir diese URL schon hatten, wars das...
        if (null!=knownURLs.get(urlStr))
        {
          Log.log("spider","url",2,"doThisUrl: already known");
          return false;
        }
        // wenn es ein Dir ist, schauen wir, ob wir das schon mal
        // als Datei hatten...
        if (urlStr.endsWith("/"))
        {
          String urlStr2=urlStr.substring(0,urlStr.length()-2);
          if (null!=knownURLs.get(urlStr2))
          {
            Log.log("spider","url",2,"doThisUrl: path already known as file");
            return false;
          }
        }
        knownURLs.put(urlStr,this);
      }
    }
    return true;	
  }
    
  // haben wir noch einen freien Runner zur Verfuegung?
  // wenn ja, bekommt er diese URL
  public boolean hasFreeSpider(URLInfo ui)
  {
    synchronized(_runner)
    {
      for (int i=0;i<_threadNum;i++)
      {
        if (null==_runner[i])
        {
          SpiderRunner sr=new SpiderRunner(ui,this);
          Log.log("spider","threads",1,"hasFreeSpider: new runner with number "+i+" has been started for "+ui._urlStr);
          _runner[i]=sr;
          sr.start();
          return true;
        }
      }
    }
    return false;
  }

  // noch weitere URLs zur Verfuegung?
  public boolean hasMoreElements()
  {
    // fertige URLs
    if (0!=done.size())
    {
      return true;
    }
    // laufende Threads
    if (runningThreads())
    {
      return true;
    }
    return false;
  }

  // naechste URL abrufen
  public URLInfo getNextElement()
  {
    // erstmal fertige URLs liefern
    synchronized (done)
    {
      // Jetzt koennen wieder Elemente eingefuegt werden
      if (done.size()<_maxQueue/2)
      {
        Log.log("spider","threads",1,"restarting thread waiting for queue");
        done.notifyAll();
      }
      if (!done.isEmpty())
      {
        URLInfo ui=(URLInfo)done.firstElement();
        done.removeElementAt(0);
        return ui;
      }
    }
    // wenn Threads laufen, warten bis was kommt...
    while (runningThreads())
    {
      Log.log("spider","threads",2,"no url in done, but running spiders...");
      checkRunner();
      Object o=new Object();
      synchronized (o)
      {
        try
        {
          o.wait(1000);
        }
        catch (InterruptedException ex) {}
        synchronized(done)
        {
          if (!done.isEmpty())
          {
            Log.log("spider","threads",3,"find one in found...");
            URLInfo ui=(URLInfo)done.firstElement();
            done.removeElementAt(0);
            return ui;
          }
        }
      }
    }
    synchronized(done)
    {
      if (!done.isEmpty())
      {
        Log.log("spider","threads",3,"find another one in found...");
        URLInfo ui=(URLInfo)done.firstElement();
        done.removeElementAt(0);
        return ui;
      }
    }
    return null;
  }
     
  // Testen, ob noch Spider laufen
  public boolean runningThreads()
  {
    for (int i=0;i<_threadNum;i++)
    {
      if (_runner[i]!=null)
      {
        Log.log("spider","threads",2,"Runner "+i+" is running...");
        return true;
      }
    }
    Log.log("spider","threads",0,"no running runners...");
    return false;
  }

  // pruefen, ob sich was tut
  // wenn nicht, wird der Thread in seiner Verbindung unterbrochen
  // und kann damit neu anfangen
  public void checkRunner()
  {
    for (int i=0;i<_threadNum;i++)
    {
      if (null!=_runner[i])
      {
        SpiderRunner r=_runner[i];
        if (!r.isAlive())
        {
          Log.log("spider","threads",1,"Thread #"+i+" for "+r.currURL+" is dead...");
          continue;
        }
        if (r.currURL!=r.compURL)
        {
          r.failCount=0;
          r.compURL=r.currURL;
          continue;
        }
        if (r.count!=r.compCount)
        {
          r.failCount=0;
          r.compCount=r.count;
          continue;
        }
        else
        {
          r.failCount++;
          if (r.failCount>300)
          {
            try
            {
              Log.log("spider","threads",0,"interrupting runner #"+i+" for "+r.currURL);
              r.stop=true;
//              ((HttpURLConnection)r.uc).disconnect();
              r.interrupt();
              r.s.close();
              r.s2.close();
              r.scanner.close();
            }
            catch (Exception ex)
            {
              ex.printStackTrace(Log.getWriter("spider","error",2));
            }
          }
        }
      }
    }
  }

  // URL ist fertig und wird in die Ergbnis-Liste eingefuegt
  public void urlIsDone(URLInfo ui)
  {
    Log.log("spider","url",0,"url is done: "+ui._urlStr);
    if (null!=ui)
    { // valid Info
      done.addElement(ui);
    } 
    else
    {
      Log.log("spider","error",1,"UrlIsDone: no urlInfo");
    }
  }

  // Runner hat nichts mehr zu tun
  public void runnerIsDone(SpiderRunner sr)
  {
    synchronized(_runner)
    {
      boolean found=false;
      for (int i=0;i<_threadNum;i++)
      {
        if (_runner[i]==sr)
        {
          Log.log("spider","threads",1,"runner "+i+" has finished");
          _runner[i]=null;
          found=true;
        }
      }
      if (!found)
      {
        Log.log("spider","error",0,"runnerIsDone: corresponding runner not found :(");
        throw new ArrayIndexOutOfBoundsException();
      }
    }
  }
  
  /// Set the authorization cookie.
  // <P>
  // Syntax is userid:password.
  public void setAuth( String auth_cookie )
  {
    this.auth_cookie = auth_cookie;
  }

 //ToDo: bessere Fehlerbehandlung mit Logging
 
  /// This method can be overridden by a subclass if you want to change
  // the broken link policy.  The default version reports the broken
  // link on the error stream.  An alternate version might attempt to
  // send mail to the owner of the page with the broken link.
  public void brokenLink( URL fromUrlStr, URL toUrlStr, String errmsg )
  {
    if (null!=fromUrlStr)
    {
      Log.log("spider","url",0,"Broken link in " + fromUrlStr.toString()+"\npointing to "+toUrlStr.toString());
    }
    else
      Log.log("spider","url",0,"error while loading " +toUrlStr.toString()+"\n"+errmsg);
  }

  /// This method can be overridden by a subclass if you want to change
  // the error reporting policy.  The default version reports the error
  // link on the error stream.  An alternate version might ignore the error.
  public void reportError( URL fromUrlStr, URL toUrlStr, String errmsg )
  {
    if (null!=fromUrlStr)
    {
      Log.log("spider","url",0,"Error in "+fromUrlStr.toString()+"\npointing to " +toUrlStr.toString());
    }
    else
      Log.log("spider","url",0,"error while loading "+toUrlStr.toString()+"\nerror: "+errmsg);
  }
  
  class Checker extends Thread
  {
    public Checker()
    {
      setDaemon(true);
    }
    
    public void run()
    {
      while (true)
      {
        Timer.waitForTick();
        checkRunner();
      }
    }
  }
  
  public Reader getReaderForURL(URL url) throws IOException
  {
    try
    {
      HTTPConnection con = HTTPConnectionPool.getConnection(url.getHost());
      HTTPResponse   rsp = con.Get(url.getFile()); 
      if (rsp.getStatusCode() >= 300)
      {
        throw new IOException ("error while receiving URL: "+rsp.getReasonLine()+new String(rsp.getData()));
      }
      InputStream s = rsp.getInputStream();
      Reader r=new InputStreamReader(s);
      return r;
    }
    catch (ModuleException ex)
    {
      throw new IOException(ex.toString());
    }
  }
  
  public InputStream getInputStreamForURL(URL url) throws IOException
  {
    try
    {
      HTTPConnection con = HTTPConnectionPool.getConnection(url.getHost());
      HTTPResponse   rsp = con.Get(url.getFile()); 
      if (rsp.getStatusCode() >= 300)
      {
        throw new IOException ("error while receiving URL: "+rsp.getReasonLine()+new String(rsp.getData()));
      }
      InputStream s = rsp.getInputStream();
      return s;
    }
    catch (ModuleException ex)
    {
      throw new IOException(ex.toString());
    }
  }
}
