/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.indexer;

import java.net.URL;
import java.util.Hashtable;

import org.w3c.dom.*;

import de.hendriklipka.ldse.server.data.document.Document;
import de.hendriklipka.ldse.server.data.document.HTMLDocument;
import de.hendriklipka.ldse.server.data.document.TextDocument;
import de.hendriklipka.ldse.server.data.document.GZipDocument;
import de.hendriklipka.ldse.server.data.document.ZipDocument;
import de.hendriklipka.ldse.server.data.document.PDFDocument;
import de.hendriklipka.ldse.server.data.URLInfo;
import de.hendriklipka.ldse.common.Log;

/**
  erzeugt eine Instanz von Document fuer eine URL
*/
public class DocumentFactory
{
  public static Document getDocument(URLInfo urlInfo)
  {
    Log.log("indexer","document",2,"trying to create document for: "+urlInfo._urlStr);
    String className=null;
    if (null!=urlInfo._type)
    {
        className=(String)_mimeClasses.get(urlInfo._type);
    }
    if ((null==className)&&(null!=urlInfo._url))
    {
        String f=urlInfo._url.getFile();
        int pos=f.lastIndexOf('.');
        if (-1!=pos)
        {
            String type=f.substring(pos+1).toLowerCase();
            if (0==type.length())
                type=".";
            className=(String)_extClasses.get(type);
        }
    }
    if (null!=className)
    {
        Class c=null;
        Object o=_classes.get(className);
        if (null==o)
        {
            try
            {
                c=Class.forName(className);
                _classes.put(className,c);
            }
            catch (Exception ex)
            {
                return null;
            }
        }
        else
        {
            c=(Class)o;
        }
        try
        {
            Object doc=c.newInstance();
            Document d=(Document)doc;
            d.init(urlInfo);
            return d;
        }
        catch (Exception ex)
        {
            return null;
        }
    }
    return null;
    /*
    if (urlInfo._type.equalsIgnoreCase("text/html"))
    {
      return new HTMLDocument(urlInfo);
    }
    else if (urlInfo._type.equalsIgnoreCase("text/plain"))
    {
      return new TextDocument(urlInfo);
    }
    else if(urlInfo._type.equalsIgnoreCase("application/x-gzip"))
    {
      return new GZipDocument(urlInfo);
    }
    else if(urlInfo._type.equalsIgnoreCase("application/zip"))
    {
      return new ZipDocument(urlInfo);
    }
    else if(urlInfo._type.equalsIgnoreCase("application/pdf"))
    {
      return new PDFDocument(urlInfo);
    }
    else // letzter Ausweg: Dateiendung
    {
      String f=urlInfo._url.getFile();
      int pos=f.lastIndexOf('.');
      if (-1!=pos)
      {
        String type=f.substring(pos+1);
        if (type.equalsIgnoreCase("txt"))
        {
          return new TextDocument(urlInfo);
        }
        else if (type.equalsIgnoreCase("text"))
        {
          return new TextDocument(urlInfo);
        }
        else if (type.equalsIgnoreCase("html"))
        {
          return new HTMLDocument(urlInfo);
        }
        else if (type.equalsIgnoreCase("htm"))
        {
          return new HTMLDocument(urlInfo);
        }
        else if (type.equalsIgnoreCase("shtm"))
        {
          return new HTMLDocument(urlInfo);
        }
        else if (type.equalsIgnoreCase("shtml"))
        {
          return new HTMLDocument(urlInfo);
        }
        else if (type.equalsIgnoreCase("asp"))
        {
          return new HTMLDocument(urlInfo);
        }
        else if (type.equalsIgnoreCase("gz"))
        {
          return new GZipDocument(urlInfo);
        }
        else if (type.equalsIgnoreCase("tgz"))
        {
          return new GZipDocument(urlInfo);
        }
        else if (type.equalsIgnoreCase("zip"))
        {
          return new ZipDocument(urlInfo);
        }
        else if (type.equalsIgnoreCase("pdf"))
        {
          return new PDFDocument(urlInfo);
        }
      }
    }
    */
  }
  
  public static void init(Node configNode)
  {
    NodeList nl=configNode.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node node=nl.item(i);
      if (node.getNodeType()!=node.ELEMENT_NODE)
        continue;
      String nn=node.getNodeName();
      if (nn.equals("documenttype"))
      {
        addType(node);
      }
    }
  }
  
  protected static void addType(Node node)
  {
    String ext=null;
    String mime=null;
    NamedNodeMap nnl=node.getAttributes();
    Node attr=nnl.getNamedItem("class");
    String className=attr.getNodeValue();
    attr=nnl.getNamedItem("mimetype");
    if (null!=attr)
        mime=attr.getNodeValue();
    attr=nnl.getNamedItem("extension");
    if (null!=attr)
        ext=attr.getNodeValue();
    addDocumentType(mime,ext,className);
  }
  
  public static void addDocumentType(String mimeType, String extension, String className)
  {
    if (null!=mimeType)
    {
        _mimeClasses.put(mimeType,className);
    }
    if (null!=extension)
    {
        _extClasses.put(extension,className);
    }
  }
  
  protected static Hashtable _mimeClasses=new Hashtable();
  protected static Hashtable _extClasses=new Hashtable();
  protected static Hashtable _classes=new Hashtable();
}